// Define the number of retry attempts for FastLED's interrupt handling
#define FASTLED_INTERRUPT_RETRY_COUNT 0

// Include the FastLED library
#include "FastLED.h"
#include "Palettes.h"
// Define LED parameters
#define LED_TYPE    WS2812B
#define DATA_PIN    4
#define COLOR_ORDER GRB
#define NUM_LEDS    70
#define BRIGHTNESS  80

// Create an array to store LED colors
CRGB leds[NUM_LEDS];

// Define the number of updates per second for FastLED
#define UPDATES_PER_SECOND 50

// Track the time since the last palette change
unsigned long lastPaletteChangeTime = 0;

int BgIndex;

// Function to fill LEDs with colors from the palette
void FillLEDsFromPaletteColors(uint8_t colorIndex) {
	uint8_t brightness = 64;

	for( int i = 0; i < NUM_LEDS; i++) {
		leds[i] = ColorFromPalette( currentPalette, colorIndex + sin8(i*16), brightness);
		colorIndex += 1;
		}
}

// Setup function, executed once at the beginning
void setup() {
  // Add LEDs to FastLED with specified parameters
  delay(1000);
  Serial.begin(115200);
  LEDS.addLeds<LED_TYPE, DATA_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalSMD5050);
  LEDS.setBrightness(BRIGHTNESS);
  Serial.println("FastLED Setup done");

  // Seed the random number generator with analogRead
  randomSeed(analogRead(0));

  BgIndex = random(gGradientPaletteCount);
  targetPalette = gGradientPalettes[BgIndex];

}

// Loop function, executed repeatedly
void loop() {
  // Get the current time
  unsigned long currentTime = millis();

  // Check if it's time to change the palette (every 10 seconds)
  if (currentTime - lastPaletteChangeTime >= 10000) {
	  BgIndex = random(gGradientPaletteCount);
	  targetPalette = gGradientPalettes[BgIndex];
    lastPaletteChangeTime = currentTime;  // Update the last change time
    Serial.println("Change palette");
  }

  // Blend the current palette toward the target palette
  uint8_t maxChanges = 24;
  nblendPaletteTowardPalette(currentPalette, targetPalette, maxChanges);

  // Adjust the start index for motion speed
  static uint8_t startIndex = 0;
  startIndex = startIndex + 1; /* motion speed */

  // Fill LEDs with colors from the current palette
  FillLEDsFromPaletteColors(startIndex);

  // Show the colors on the LEDs
  FastLED.show();

  // Delay for the specified updates per second
  FastLED.delay(1000 / UPDATES_PER_SECOND);
}


